<?php

/**
  * Cookie class, Cookie.php
  * Cookies management
  * @category classes
  *
  * @author PrestaShop <support@prestashop.com>
  * @copyright PrestaShop
  * @license http://www.opensource.org/licenses/osl-3.0.php Open-source licence 3.0
  * @version 0.8
  *
  */

class	Cookie
{
	/** @var array Contain cookie content in a key => value format */
	private $_content;
	
	/** @var array Crypted cookie name for setcookie() */
	private $_name;
	
	/** @var array expiration date for setcookie() */
	private $_expire;
	
	/** @var array Website domain for setcookie() */
	private $_domain;
	
	/** @var array Path for setcookie() */
	private $_path;
	
	/** @var array Blowfish instance */
	private $_bf;
	
	/** @var array 56 chars Blowfish initialization key */
	private $_key;
	
	/** @var array 8 chars Blowfish initilization vector */
	private $_iv;
	
	/**
	  * Get data if the cookie exists and else initialize an new one
	  *
	  * @param $name Cookie name before encrypting
	  * @param $path 
	  */
	function __construct($name, $path = '')
	{
		$this->_content = array();
		$this->_name = md5($name._COOKIE_KEY_);
		$this->_expire = 0;
		$this->_path = trim(__PS_BASE_URI__.$path, '/\\').'/';
		if ($this->_path[0] != '/') $this->_path = '/'.$this->_path;
		$this->_key = _COOKIE_KEY_;
		$this->_iv = _COOKIE_IV_;
		$this->_domain = ($_SERVER['HTTP_HOST'] == 'localhost') ? false : ereg_replace('^[^\.]*\.([^\.]*)\.(.*)$', '\1.\2', $_SERVER['HTTP_HOST']);
		$this->_bf = new Blowfish($this->_key, $this->_iv);
		$this->update();
	}
	
	/**
	  * Set expiration date
	  *
	  * @param integer $expire Expiration time from now
	  */
	function setExpire($expire)
	{
		$this->_expire = intval($expire);
	}
	
	/**
	  * Magic method wich return cookie data from _content array
	  *
	  * @param $key key wanted
	  * @return string value corresponding to the key 
	  */
	function __get($key)
	{
		return isset($this->_content[$key]) ? $this->_content[$key] : false;
	}
	
	/**
	  * Magic method wich check if key exists in the cookie
	  *
	  * @param $key key wanted
	  * @return boolean key existence
	  */
	function __isset($key)
	{
		return isset($this->_content[$key]);
	}
	
	/**
	  * Magic method wich add data into _content array
	  *
	  * @param $key key desired
	  * @param $value value corresponding to the key 
	  */
	function __set($key, $value)
	{
		if (ereg('|\|', $key) OR ereg('|\|', $value))
			throw new Exception('Forbidden chars in cookie');
		$this->_content[$key] = $value;
		$this->write();
	}
	
	/**
	  * Magic method wich delete data into _content array
	  *
	  * @param $key key wanted
	  */
	function __unset($key)
	{
		unset($this->_content[$key]);
		$this->write();
	}
	
	/**
	  * Check customer informations saved into cookie and return customer validity
	  *
	  * @return boolean customer validity
	  */
	function isLogged()
	{
		/* Customer is valid only if it can be load and if cookie password is the same as database one */
	 	if (isset($this->logged)
				AND	$this->logged == 1
				AND isset($this->id_customer)
				AND Validate::isUnsignedId($this->id_customer) 
				AND Customer::checkPassword(intval($this->id_customer), $this->passwd))
        	return true;
        return false;
	}

	/**
	  * Check employee informations saved into cookie and return employee validity
	  *
	  * @return boolean employee validity
	  */
	function isLoggedBack()
	{
		/* Employee is valid only if it can be load and if cookie password is the same as database one */
	 	if (isset($this->id_employee)
				AND Validate::isUnsignedId($this->id_employee) 
				AND Employee::checkPassword(intval($this->id_employee), $this->passwd))
			return true;
		return false;
	}
	
	/**
	  * Delete cookie
	  */
	function logout()
	{
		$this->_content = array();
		$this->_setcookie();
		unset($_COOKIE[$this->_name]);
	}
	
	/**
	  * Soft logout, delete everything links to the customer
	  * but leave there affiliate's informations
	  */
	function mylogout()
	{
		unset($this->_content['id_customer']);
		unset($this->_content['customer_name']);
		unset($this->_content['customer_surname']);
		unset($this->_content['passwd']);
		unset($this->_content['logged']);
		unset($this->_content['email']);
		$this->write();
	}
	
	/**
	  * Get cookie content
	  */
	function update($nullValues = false)
	{
		if (isset($_COOKIE[$this->_name]))
		{
			/* Decrypt cookie content */
			$content = $this->_bf->decrypt($_COOKIE[$this->_name]);
			
			/* Get cookie checksum */
			$checksum = crc32($this->_iv.substr($content, 0, strrpos($content, '') + 1));
			
			/* Unserialize cookie content */
			$tmpTab = explode('', $content);
			foreach ($tmpTab AS $keyAndValue)
			 {
				 $tmpTab2 = explode('|', $keyAndValue);
				 if (sizeof($tmpTab2) == 2)
					 $this->_content[$tmpTab2[0]] = $tmpTab2[1];
			 }
			
			/* Check if cookie has not been modified */
			if (!isset($this->_content['checksum']) OR $this->_content['checksum'] != $checksum)
				die(Tools::displayError());
		}
		else
			$this->_content['date_add'] = date('Y-m-d H:i:s');
	}
	
	/**
	  * Setcookie according to php version
	  */
	private function _setcookie($cookie = NULL)
	{
		if ($cookie)
		{
			$content = $this->_bf->encrypt($cookie);
			$time = $this->_expire;
		}
		else
		{
			$content = 0;
			$time = time() - 1;			
		}
		if (version_compare(phpversion(), '5.2.0') == -1)
			return setcookie($this->_name, $content, $time, $this->_path, $this->domain, 0);
		else
			return setcookie($this->_name, $content, $time, $this->_path, $this->domain, 0, true);
	}
	
	/**
	  * Save cookie with setcookie()
	  */
	function write()
	{
		$cookie = '';
		
		/* Serialize cookie content */
		if (isset($this->_content['checksum'])) unset($this->_content['checksum']);
		foreach ($this->_content AS $key => $value)
			$cookie .= $key.'|'.$value.'';
			
		/* Add checksum to cookie */
		$cookie .= 'checksum|'.crc32($this->_iv.$cookie);
		
		/* Cookies are encrypted for evident security reasons */
		return $this->_setcookie($cookie);
	}

}

?>